/**
 * 
 */
package gentree;

import draw.Circle;
import draw.ICanvas;
import draw.Line;

/**
 * @author fennell
 * 
 */
public class InnerNode implements ITree {

	private final ITree left;
	private final ITree right;
	private final int data;

	/**
	 * @param left
	 *            left subtree
	 * @param right
	 *            right subtree
	 * @param data
	 *            the payload
	 */
	public InnerNode(int data, ITree left, ITree right) {
		super();
		this.left = left;
		this.right = right;
		this.data = data;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see trees.ITree#height()
	 */
	@Override
	public int height() {
		return 1 + Math.max(left.height(), right.height());
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see trees.ITree#draw(int, int, int, int, draw.ICanvas)
	 */
	@Override
	public void draw(int x, int y, int width, int height, ICanvas canvas) {
		if (!Placement.nodeFits(Placement.NODE_SIZE, width, height)) {
			canvas.abort("Inner Node " + data + " does not fit on the canvas: " + x + ", " + y + ", " + width + ", " + height);
		} else {
			int centerX = x + width/2;
			int centerY = y + Placement.NODE_SIZE/2;
			int leftX = x; 
			int rightX = centerX;
			int subY = y + Placement.NODE_DISTANCE;
			int subWidth = width/2;
			int subHeight = height - Placement.NODE_DISTANCE;
			canvas.drawCircle(new Circle(centerX, centerY, Placement.NODE_SIZE/2, "" + data));
			this.left.draw(leftX, subY, subWidth, subHeight, canvas);
			this.right.draw(rightX, subY, subWidth, subHeight, canvas);
			canvas.drawLine(new Line(centerX, centerY, leftX + subWidth/2, subY));
			canvas.drawLine(new Line(centerX, centerY, rightX + subWidth/2, subY));
		}
	}
}
